package kr.co.vividnext.sodalive.useraction

import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import kr.co.vividnext.sodalive.fcm.FcmService
import kr.co.vividnext.sodalive.point.MemberPoint
import kr.co.vividnext.sodalive.point.MemberPointRepository
import kr.co.vividnext.sodalive.point.PointGrantLog
import kr.co.vividnext.sodalive.point.PointGrantLogRepository
import kr.co.vividnext.sodalive.point.PointRewardPolicyRepository
import org.springframework.stereotype.Service
import java.time.LocalDateTime

@Service
class UserActionService(
    private val repository: UserActionLogRepository,
    private val policyRepository: PointRewardPolicyRepository,
    private val grantLogRepository: PointGrantLogRepository,
    private val memberPointRepository: MemberPointRepository,

    private val fcmService: FcmService
) {

    private val coroutineScope = CoroutineScope(Dispatchers.IO)

    fun recordAction(memberId: Long, actionType: ActionType, pushToken: String?) {
        coroutineScope.launch {
            val now = LocalDateTime.now()
            repository.save(UserActionLog(memberId, actionType))

            val policy = policyRepository.findByActionTypeAndIsActiveTrue(actionType, now)
            if (policy != null) {
                val actionCount = repository.countByMemberIdAndActionTypeAndCreatedAtBetween(
                    memberId = memberId,
                    actionType = actionType,
                    startDate = policy.startDate,
                    endDate = policy.endDate ?: now
                )
                if (actionCount < policy.threshold) return@launch

                val alreadyGranted = grantLogRepository.existsByMemberIdAndPolicyId(memberId, policy.id!!)
                if (alreadyGranted) return@launch

                memberPointRepository.save(
                    MemberPoint(
                        memberId = memberId,
                        point = policy.pointAmount,
                        actionType = actionType,
                        expiresAt = now.plusDays(3)
                    )
                )

                grantLogRepository.save(
                    PointGrantLog(
                        memberId = memberId,
                        point = policy.pointAmount,
                        actionType = actionType,
                        policyId = policy.id!!
                    )
                )

                if (pushToken != null) {
                    fcmService.sendPointGranted(pushToken, policy.pointAmount)
                }
            }
        }
    }
}
